/*
 * Teste de blink modificado.
 * Pisca ambos os LEDs em threads separadas.
 * Realiza log de uma mensagem para o RTT toda vez que o LED Azul muda de estado.
 */
#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <logging/log.h>
/* 1000 msec = 1 sec */
#define SLEEP_LED0_TIME_MS 500
#define SLEEP_LED1_TIME_MS 500

/* Ativar o link dos apelidos na descricao da board foobar com nRF52840*/
#define LED0_NODE DT_ALIAS(led1)
#define LED1_NODE DT_ALIAS(led0)

#if DT_NODE_HAS_STATUS(LED1_NODE, okay)
#define LED1 DT_GPIO_LABEL(LED1_NODE, gpios)
#define PIN1 DT_GPIO_PIN(LED1_NODE, gpios)
#define FLAGS1 DT_GPIO_FLAGS(LED1_NODE, gpios)
#else
/* A build error here means your board isn't set up to blink an LED. */
#error "Unsupported board: led0 devicetree alias is not defined"
#define LED1 ""
#define PIN1 0
#define FLAG1 0
#endif

#if DT_NODE_HAS_STATUS(LED0_NODE, okay)
#define LED0 DT_GPIO_LABEL(LED0_NODE, gpios)
#define PIN0 DT_GPIO_PIN(LED0_NODE, gpios)
#define FLAGS0 DT_GPIO_FLAGS(LED0_NODE, gpios)
#else
/* A build error here means your board isn't set up to blink an LED. */
#error "Unsupported board: led0 devicetree alias is not defined"
#define LED0 ""
#define PIN 0
#define FLAGS 0
#endif

// Tamanho na stack reservado para cada thread
#define STACKSIZE 500
#define PRIORITY 5

// Funcoes para rodar em threads
extern void led0_control(void *, void *, void *);
extern void led1_control(void *, void *, void *);

void main(void)
{
    while (1)
    {
        k_msleep(SLEEP_LED0_TIME_MS);
    }
}

// Controle do LED verde na placa
void led0_control(void *p1, void *p2, void *p3)
{
    const struct device *dev;
    int ret;
    // Resgate do struct do dispositivo com o nome providenciado na board
    dev = device_get_binding(LED0);
    if (dev == NULL)
    {
        return;
    }

    // Ativa a saida como nível lógico alto usando o HAL do Zephyr
    ret = gpio_pin_configure(dev, PIN0, GPIO_OUTPUT_ACTIVE | FLAGS0);
    if (ret < 0)
    {
        return;
    }

    // Inverter o estado do pin usando o HAL do Zephyr a cada intervalo de tempo
    // com o sleep do kernel Zephyr
    while (1)
    {
        gpio_pin_toggle(dev, PIN0);
        k_msleep(SLEEP_LED0_TIME_MS);
    }
}

// Controle do LED azul na placa
void led1_control(void *p1, void *p2, void *p3)
{
    const struct device *dev;
    bool led_is_on = true;
    int ret;
    dev = device_get_binding(LED1);
    if (dev == NULL)
    {
        return;
    }

    ret = gpio_pin_configure(dev, PIN1, GPIO_OUTPUT_ACTIVE | FLAGS1);
    if (ret < 0)
    {
        return;
    }

    // Dar set no GPIO, invertendo flag booleana
    // a cada ciclo dado pelo intervalo de tempo
    while (1)
    {
        gpio_pin_set(dev, PIN1, (int)led_is_on);
        led_is_on = !led_is_on;
        // Print para o console RTT da Segger, configurado em prj.conf
        printk("BLUE LED STATUS: %d \n", led_is_on);
        k_msleep(SLEEP_LED1_TIME_MS);
    }
}

// Macro que define as threads em tempo de compilação

// Controla o LED Verde
K_THREAD_DEFINE(led0_control_id, STACKSIZE, led0_control, NULL, NULL, NULL,
                PRIORITY, 0, 0);

// Controla o LED Azul
K_THREAD_DEFINE(
    led1_control_id,  // Identificador da thread
    STACKSIZE,        // Tamanho da Thread
    led1_control,     // Função de entrypoint a ser executada pela Thread
    NULL, NULL, NULL, // Argumentos p1,p2,p3 para a função de entrypoint
    PRIORITY,         // Prioridade da Thread
    0,                // Opções
    0                 // Delay para escalonamento
);
